import urlparse
from waunit_utils import *

def parse_google_analytics_url(url):
    """
    Parse Google Analytics URL
    """
    unit_test_data={}
    parsed = urlparse.urlparse(url)
    queryparams = urlparse.parse_qs(parsed.query)
    cacheBuster = to_string(queryparams.get('z'))       # Cache buster
    hitType = to_string(queryparams.get('t'))         # Pageview or event
    pageTitle = to_string(queryparams.get('dt'))        # pageTitle
    pageUrl = to_string(queryparams.get('dl'))          # Full URL
    propertyId = to_string(queryparams.get('tid'))      # UA-XXXXXX-X
    eventLabel = to_string(queryparams.get('el'))       # Event Label
    eventAction = to_string(queryparams.get('ea'))      # Event Action
    eventCategory = to_string(queryparams.get('ec'))    # Event Category

    # To uniquely identify each test we add a query param utid=
    # and read it from the document location.
    parsed = urlparse.urlparse(pageUrl)
    queryparams = urlparse.parse_qs(parsed.query)
    # Unit Test ID
    utid = to_string(queryparams.get('utid'))
    unit_test_data[utid] = {'hitType': hitType,
                    'pageTitle': pageTitle,
                    'pageUrl' : pageUrl,
                    'eventCategory' : eventCategory,
                    'eventAction' : eventAction,
                    'eventLabel' : eventLabel,
                    'propertyId':propertyId
                    }
    return unit_test_data






def to_string(list):
    """
    Convert list to string. If empty it is set to (not set)
    """
    if list:
      return ''.join(list)
    else:
      return '(not set)'



def verify_spec(spec_utid, proxy_utid):
  """
  For a specific unit test id (utid) compares the spec with the proxy
  """
  results=''
  for key in spec_utid:
    results += '%s: spec=%s, proxy=%s (%s) *** ' % (key,spec_utid[key],proxy_utid[key],(spec_utid.get(key)==proxy_utid.get(key)))
  return results




#!/usr/bin/env python
"""
    mitmproxy Proxy
"""

from WAUnit import *

from mitmproxy import flow
from mitmproxy.proxy import ProxyServer, ProxyConfig
import shelve
import ConfigParser

#config = ConfigParser.RawConfigParser()
#config.read('wunit.cfg')

import os.path

# Save Parsed Files
import argparse
parser = argparse.ArgumentParser(description='Run the Web Analytics Unit Proxy')
parser.add_argument('--config', dest='config_file',
                       help='specified the configuration file')
args = parser.parse_args()

global config
config = ConfigParser.RawConfigParser()
config.read(args.config_file)

def MainProxy():
    """
    The Proxy called by waunitproxy
    """
    # Configuration file
    if(os.path.isfile(args.config_file)):
    #use ~/.mitmproxy/mitmproxy-ca.pem as default CA file.
        proxyconfig = ProxyConfig(port = config.getint('Proxy', 'port'),
        cadir = "~/.mitmproxy/")
        state = flow.State()
        server = ProxyServer(proxyconfig)
        try:
            m = GoogleAnalyticsUniversal(server, state)
            m.run()
        except (KeyboardInterrupt):
            pass
    else:
        print "Invalid config file. Use --config to specify the configuration file (e.g. --config waunit.cfg)"


class GoogleAnalyticsUniversal(flow.FlowMaster):


    def run(self):
        try:
            print "Running..."
            flow.FlowMaster.run(self)
        except KeyboardInterrupt:
            self.shutdown()

    def handle_request(self, f):
        f = flow.FlowMaster.handle_request(self, f)
        if f:
            f.reply()
        return f

    def handle_response(self, f):
        f = flow.FlowMaster.handle_response(self, f)
        if f:
            f.reply()
        if "collect" in f.request.path:
            d = shelve.open(config.get('Proxy', 'hitsdb'))
            url = "%s://%s%s" % (f.request.scheme, f.request.host, f.request.path)
            ut = parse_google_analytics_url(url)
            print '\n'
            print ut
            d[to_string(ut.keys())] = ut[to_string(ut.keys())]
            d.close()
        return f



